/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XForms support.
 *
 * The Initial Developer of the Original Code is
 * IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Brian Ryner <bryner@brianryner.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef nsXFormsStubElement_h_
#define nsXFormsStubElement_h_

#include "nsIXTFGenericElement.h"
#include "nsIXTFXMLVisual.h"
#include "nsIXTFBindableElement.h"
#include "nsIXFormsControlBase.h"

/**
 * An implementation of a generic non-UI XForms element.
 * 
 * This is the implementation for e.g. bind so that the id attribute works 
 * correctly for these nodes.  It is also a base class for all XForms
 * nsIXTFGenericElement implementations.
 */
class nsXFormsStubElement : public nsIXTFGenericElement
{
protected:
  // We need a virtual destructor so that when a subclass does
  // NS_IMPL_ISUPPORTS_INHERITED, our Release() implementation calls the
  // derived class destructor.
  virtual ~nsXFormsStubElement() {}

public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIXTFELEMENT
  NS_DECL_NSIXTFGENERICELEMENT
};

/**
 * An implementation of an XForms UI control.
 *
 * This is a base class for UI controls that use nsIXTFXMLVisual.
 * Note: a pointer to the anonymous content root is _not_ provided as part
 * of this base class because different implementations want to store
 * different types of interface pointers.
 */
class nsXFormsXMLVisualStub : public nsIXTFXMLVisual
{
protected:
  // We need a virtual destructor so that when a subclass does
  // NS_IMPL_ISUPPORTS_INHERITED, our Release() implementation calls the
  // derived class destructor.
  virtual ~nsXFormsXMLVisualStub() {}

public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIXTFELEMENT
  NS_DECL_NSIXTFVISUAL
  NS_DECL_NSIXTFXMLVISUAL
};

/**
 * An implementation of an XForms bindable control.
 *
 * This is a base class for UI controls that use nsIXTFBindableElement.
 * Note: a pointer to the anonymous content root is _not_ provided as part
 * of this base class because different implementations want to store
 * different types of interface pointers.
 */
class nsXFormsBindableStub : public nsIXTFBindableElement
{
protected:
  // We need a virtual destructor so that when a subclass does
  // NS_IMPL_ISUPPORTS_INHERITED, our Release() implementation calls the
  // derived class destructor.
  virtual ~nsXFormsBindableStub() {}

public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIXTFELEMENT
  NS_DECL_NSIXTFBINDABLEELEMENT
};

/* Factory methods */
NS_HIDDEN_(nsresult)
NS_NewXFormsStubElement(nsIXTFElement **aResult);

NS_HIDDEN_(nsresult)
NS_NewXFormsXMLVisualStub(nsIXTFElement **aResult);

NS_HIDDEN_(nsresult)
NS_NewXFormsBindableStub(nsIXTFElement **aResult);
#endif
