// Cbrowse.idl : IDL source for Cbrowse.exe

import "oaidl.idl";

typedef enum 
{
	trNotRun,
	trFailed,
	trPassed,
	trPartial
} TestResult;

[
	object,
	uuid(95AF1AB1-FA66-11D2-A284-000000000000),
	dual,
	helpstring("DITestScriptHelper Interface"),
	pointer_default(unique)
]
interface DITestScriptHelper : IDispatch
{
	[id(1), helpstring("method OutputString")] HRESULT OutputString(BSTR bstrMessage);
	[propget, id(2), helpstring("property WebBrowser")] HRESULT WebBrowser([out, retval] LPDISPATCH *pVal);
	[propput, id(3), helpstring("property Result")] HRESULT Result([in] TestResult newVal);
	[propget, id(4), helpstring("property TestURL")] HRESULT TestURL([out, retval] BSTR *pVal);
	[propget, id(5), helpstring("property TestCGI")] HRESULT TestCGI([out, retval] BSTR *pVal);
};

[
	uuid(5B209213-F756-11D2-A27F-000000000000),
	version(1.0),
	helpstring("Cbrowse 1.0 Type Library")
]
library CbrowseLib
{
	importlib("stdole32.tlb");
	importlib("stdole2.tlb");
	[
		uuid(95AF1AB2-FA66-11D2-A284-000000000000),
		helpstring("TestScriptHelper Class")
	]
	coclass TestScriptHelper
	{
		[default] interface DITestScriptHelper;
	};
	[
		uuid(31204F42-FCE8-11D2-A289-000000000000),
		helpstring("ControlEventSink Class")
	]
	coclass ControlEventSink
	{
		[default] interface IDispatch;
	};
	[
		uuid(8EC06081-21DE-11D3-941E-000000000000),
		helpstring("CBrowserCtlSite Class")
	]
	coclass CBrowserCtlSite
	{
		[default] interface IUnknown;
	};
};

	