/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 *  Zack Rusin <zack@kde.org>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Lars Knoll <knoll@kde.org>
 *   Zack Rusin <zack@kde.org>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
#include "EmbedWindowCreator.h"

#include "qgeckoembed.h"
#include "qgeckoglobals.h"
#include "EmbedWindow.h"


EmbedWindowCreator::EmbedWindowCreator()
{
}

EmbedWindowCreator::~EmbedWindowCreator()
{
}

NS_IMPL_ISUPPORTS1(EmbedWindowCreator, nsIWindowCreator)

NS_IMETHODIMP
EmbedWindowCreator::CreateChromeWindow(nsIWebBrowserChrome *aParent,
                                       PRUint32 aChromeFlags,
                                       nsIWebBrowserChrome **_retval)
{
    NS_ENSURE_ARG_POINTER(_retval);

    QGeckoEmbed *newEmbed = 0;

    // No parent?  Ask via the singleton object instead.
    if (!aParent) {
        //create single create window
        qDebug("XXXXXXXXXXXXXXXXXXXXXXXXXXXXX not implemented");
    }
    else {
        // Find the QGeckoEmbed object for this web browser chrome object.
        QGeckoEmbed *qecko = QGeckoGlobals::findPrivateForBrowser(aParent);

        if (!qecko)
            return NS_ERROR_FAILURE;
        newEmbed = qecko;
        emit newEmbed->newWindow(newEmbed, aChromeFlags);
    }

    // check to make sure that we made a new window
    if (!newEmbed)
        return NS_ERROR_FAILURE;

    qDebug("MMMMM here");
    // set the chrome flag on the new window if it's a chrome open
    if (aChromeFlags & nsIWebBrowserChrome::CHROME_OPENAS_CHROME)
        newEmbed->setIsChrome(PR_TRUE);

    *_retval = NS_STATIC_CAST(nsIWebBrowserChrome *,
                              (newEmbed->window()));

    if (*_retval) {
        NS_ADDREF(*_retval);
        return NS_OK;
    }

    return NS_ERROR_FAILURE;
}
