calendar-todo-list {
    -moz-binding: url("calendar-todo-list.xml#calendar-todo-list");
}

calendar-todo-item {
    -moz-binding: url("calendar-todo-list.xml#calendar-todo-item");
}

.calendar-todo-item-label, .calendar-todo-item-checkbox {
    -moz-box-align: center;
    margin: 0px;
}
