/* Skin-domain style rules for lightning UI (pinstripe version) */

calendar-todo-list {
    background-color: -moz-Field;
    color: -moz-FieldText;
}

calendar-todo-list[disabled="true"] {
    color: GrayText;
}

calendar-todo-item {
  -moz-box-align: center;
  margin-top: 0;
  margin-bottom: 0;
}

calendar-todo-item[selected="true"] {
  background-color: Highlight;
  color: HighlightText;
}

treechildren::-moz-tree-image(col-calendar-Checkbox, checked) {
   list-style-image: url("chrome://calendar/skin/checkbox_checked.png");
}

treechildren::-moz-tree-image(col-calendar-Checkbox, unchecked) {
   list-style-image: url("chrome://calendar/skin/checkbox_unchecked.png");
}

/*--------------------------------------------------------------------
 * Navigation controls for the views
 *-------------------------------------------------------------------*/

.nav-buttons{
    list-style-image: url("chrome://calendar/skin/prevnextarrow.png");
}

.nav-buttons[direction="next"] {
    -moz-box-align:end;
    -moz-image-region: rect(18px 18px 36px 0px);
    cursor:pointer;
}

.next-button:hover:active {
    margin: 6px 14px 4px 16px;
}

.nav-buttons[direction="prev"] {
    -moz-box-align:start;
    -moz-image-region: rect(0px 18px 18px 0px);
    cursor:pointer;
}

.prev-button:hover:active {
    margin: 6px 14px 4px 16px;
}

.title-container {
    -moz-box-align: center;
    -moz-box-pack: center;
    margin: 0px 15px 0px 15px;
}

.title2 {
    font-size:1.2em;
    color: #4e84c2;
    margin:0px;
    padding:0px;
    cursor:pointer;
    -moz-appearance: none;
    border: 0px;
}

.title1 {
    font-size:1.5em;
    color: #4e84c2;
    cursor:pointer;
    -moz-appearance: none;
    border: 0px;
}

.title0 {
    font-size:2.0em;
    color: #2e4e73;
    font-weight:bold;
    margin:0px;
    padding:0px;
    -moz-appearance: none;
    border: 0px;
}

.view-title-label-box
{
    width: 10em !important;
    -moz-box-align: center;
    -moz-box-pack: end;
}
